@@ -9,30 +9,23 @@ from msmart.command import base_command as request_status_command
 from msmart.command import set_command
 from msmart.packet_builder import packet_builder
 
-VERSION = '0.1.25'
+VERSION = '0.1.26'
 
 _LOGGER = logging.getLogger(__name__)
 
 
 def convert_device_id_hex(device_id: int):
-    hex_string = hex(device_id)[2:]
-    if len(hex_string) % 2 != 0:
-        hex_string = '0' + hex_string
-    old = bytearray.fromhex(hex_string)
-    new = reversed(old)
-    return bytearray(new).hex()
+    return device_id.to_bytes(6, 'little').hex()
 
 
 def convert_device_id_int(device_id: str):
-    old = bytearray.fromhex(device_id)
-    new = reversed(old)
-    return int(bytearray(new).hex(), 16)
+    return int.from_bytes(bytes.fromhex(device_id), 'little')
 
 
 class device:
 
     def __init__(self, device_ip: str, device_id: int, device_port: int):
-        device_id = convert_device_id_hex(device_id)
+        # device_id = convert_device_id_hex(device_id)
         self._lan_service = lan(device_ip, device_id, device_port)
         self._ip = device_ip
         self._id = device_id
@@ -45,11 +38,18 @@ class device:
         self._support = False
         self._online = True
         self._active = True
+        self._protocol_version = 2
 
-    def setup(self):
-        # self.air_conditioning_device.refresh()
-        device = air_conditioning_device(self._ip, self._id, self._port)
-        return device
+    def authenticate(self, key: str, token: str):
+        # compatible example.py
+        if key != "YOUR_AC_K1" and token != "YOUR_AC_TOKEN":
+            self._protocol_version = 3
+            self._token = bytearray.fromhex(token)
+            self._key = bytearray.fromhex(key)
+            self._authenticate()
+
+    def _authenticate(self):
+        self._lan_service.authenticate(self._token, self._key)
 
     def set_device_detail(self, device_detail: dict):
         self._id = device_detail['id']
@@ -74,6 +74,10 @@ class device:
     def ip(self):
         return self._ip
 
+    @property
+    def port(self):
+        return self._ip
+
     @property
     def name(self):
         return self._name
@@ -115,6 +119,7 @@ class air_conditioning_device(device):
 
     class fan_speed_enum(Enum):
         Auto = 102
+        Full = 100
         High = 80
         Medium = 60
         Low = 40
@@ -166,8 +171,8 @@ class air_conditioning_device(device):
             _LOGGER.debug("Unknown Swing Mode: {}".format(value))
             return air_conditioning_device.swing_mode_enum.Off
 
-    def __init__(self, device_ip: str, device_id: str, device_port: int):
-        super().__init__(device_ip, convert_device_id_int(device_id), device_port)
+    def __init__(self, *args, **kwargs):
+        super(air_conditioning_device, self).__init__(*args, **kwargs)
         self._prompt_tone = False
         self._power_state = False
         self._target_temperature = 17.0
@@ -176,7 +181,7 @@ class air_conditioning_device(device):
         self._swing_mode = air_conditioning_device.swing_mode_enum.Off
         self._eco_mode = False
         self._turbo_mode = False
-        self.farenheit_unit = False # default unit is Celcius. this is just to control the temperatue unit of the AC's display. the target_temperature setter always expects a celcius temperature (resolution of 0.5C), as does the midea API
+        self.farenheit_unit = False  # default unit is Celcius. this is just to control the temperatue unit of the AC's display. the target_temperature setter always expects a celcius temperature (resolution of 0.5C), as does the midea API
 
         self._on_timer = None
         self._off_timer = None
@@ -187,15 +192,29 @@ class air_conditioning_device(device):
 
     def refresh(self):
         cmd = request_status_command(self.type)
+        self._send_cmd(cmd)
+
+    def _send_cmd(self, cmd):
         pkt_builder = packet_builder(self.id)
         pkt_builder.set_command(cmd)
-
         data = pkt_builder.finalize()
-        data = self._lan_service.appliance_transparent_send(data)
+        if self._protocol_version == 3:
+            responses = self._lan_service.appliance_transparent_send_8370(data)
+        else:
+            responses = self._lan_service.appliance_transparent_send(data)
+        for response in responses:
+            self._process_response(response)
+
+    def _process_response(self, data):
         _LOGGER.debug(
-            "refresh - Recieved from {}, {}: {}".format(self.ip, self.id, data.hex()))
+            "Update from {}:{} {}".format(self.ip, self.port, data.hex()))
         if len(data) > 0:
             self._online = True
+            if data == b'ERROR':
+                _LOGGER.debug(
+                    "got ERROR from {}, {}".format(self.ip, self.id))
+                # self._authenticate()
+                return
             response = appliance_response(data)
             self._defer_update = False
             self._support = True
@@ -204,8 +223,8 @@ class air_conditioning_device(device):
                     self.update(response)
                 if data[0xa] == 0xa1 or data[0xa] == 0xa0:
                     '''only update indoor_temperature and outdoor_temperature'''
-                    _LOGGER.debug("refresh - Special Respone. {}, {}: {}".format(
-                        self.ip, self.id, data[0xa:].hex()))
+                    _LOGGER.debug("Update - Special Respone. {}:{} {}".format(
+                        self.ip, self.port, data[0xa:].hex()))
                     pass
                     # self.update_special(response)
                 self._defer_update = False
@@ -224,30 +243,10 @@ class air_conditioning_device(device):
             cmd.swing_mode = self._swing_mode.value
             cmd.eco_mode = self._eco_mode
             cmd.turbo_mode = self._turbo_mode
-            pkt_builder = packet_builder(self.id)
+            # pkt_builder = packet_builder(self.id)
 #            cmd.night_light = False
             cmd.fahrenheit = self.farenheit_unit
-            pkt_builder.set_command(cmd)
-
-            data = pkt_builder.finalize()
-            data = self._lan_service.appliance_transparent_send(data)
-            _LOGGER.debug(
-                "apply - Recieved from {}, {}: {}".format(self.ip, self.id, data.hex()))
-            if len(data) > 0:
-                self._online = True
-                response = appliance_response(data)
-                self._support = True
-                if not self._defer_update:
-                    if data[0xa] == 0xc0:
-                        self.update(response)
-                    if data[0xa] == 0xa1 or data[0xa] == 0xa0:
-                        '''only update indoor_temperature and outdoor_temperature'''
-                        _LOGGER.debug("apply - Special Respone. {}, {}: {}".format(
-                            self.ip, self.id, data[0xa:].hex()))
-                        pass
-                        # self.update_special(response)
-            elif not self._keep_last_known_online_state:
-                self._online = False
+            self._send_cmd(cmd)
         finally:
             self._updating = False
             self._defer_update = False
@@ -263,15 +262,15 @@ class air_conditioning_device(device):
             res.swing_mode)
         self._eco_mode = res.eco_mode
         self._turbo_mode = res.turbo_mode
-        indoor_temperature = res.indoor_temperature 
+        indoor_temperature = res.indoor_temperature
         if indoor_temperature != 0xff:
             self._indoor_temperature = indoor_temperature
         outdoor_temperature = res.outdoor_temperature
         if outdoor_temperature != 0xff:
             self._outdoor_temperature = outdoor_temperature
-        self._timer_on = res.on_timer
-        self._timer_off = res.off_timer
-    
+        self._on_timer = res.on_timer
+        self._off_timer = res.off_timer
+
     def update_special(self, res: appliance_response):
         indoor_temperature = res.indoor_temperature
         if indoor_temperature != 0xff:
